/******************************************************************************/
/*                                                                            */
/* qml.c                                                                      */
/*                                                                            */
/*                                                                            */
/* maximum likelihood computation for quartets                                */
/*                                                                            */
/* by Korbinian Strimmer and Arndt von Haeseler (1995)                        */
/*                                                                            */
/*                                                                            */
/* the computational engine is taken from the                                 */
/* maximum likelihood program NucML/ProtML V2.2                               */
/* by Jun Adachi and Masami Hasegawa (1994)                                   */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
/* definitions and prototypes                                                 */
/******************************************************************************/


#include "qml.h"
#include <stdlib.h>
#include <math.h>
#include "matrixut.h"

#define TRUE 1
#define FALSE 0

#define MAXARC     300.0    /* upper limit on branch length (substitutions) */
#define MIDARC     100.0
#define MINARC     0.001    /* lower limit on branch length (substitutions) */
#define EPSILON    0.001    /* stopping value of error (branch length)      */
#define DEPSILON   0.0001   /* stopping value of error (distance)           */
#define MAXIT      15       /* maximum number of iterates of smoothing      */
#define MINFREQ    0.005    /* lower limit on nucleotide frequencies        */
#define NUMSPC     4        /* number of taxa in a quartet                  */
#define NUMIBRNCH  1        /* number of internal branches in a quartet     */
#define NUMBRNCH   5        /* number of branches in a quartet              */
#define INFILE     "infile" /* name of sequence input file                  */

typedef struct node {
	struct node *isop;
	struct node *kinp;
	int descen;
	int number;
	double length;
	double lklhdl;
	ivector paths;
	ivector eprob;
	dmatrix iprob;
} Node;

typedef struct tree {
	Node *rootp;
	Node **ebrnchp;
	Node **ibrnchp;
	double lklhd;
} Tree;

FILE *seqfp;       /* pointer to sequence input file */
Tree *Ctree;       /* pointer to quartet tree */
int Numptrn;     
imatrix Seqconint;
ivector Weight;
char **Identif;    /* species names   */
cmatrix Seqchar;   /* sequence data   */
dmatrix Distanmat; /* distance matrix */
dvector Distanvec;
dvector Brnlength;
dvectpmty Freqemp;
dvectpmty Eval;
dvectpmty Evl2;
dmattpmty Evec;
dmattpmty Ievc;

void getsizesites(FILE *ifp); /* getseq.c  */
void getdataset(FILE *ifp);
void evaluateseqs();          /* seqproc.c */
void computedistan();         /* distan.c  */
void tranprobmat();           /* tranprb.c */
void tprobmtrx(double arc, dmattpmty tpr);
void tdiffmtrx(double arc, dmattpmty tpr, dmattpmty td1, dmattpmty td2);
void truelkl(Tree *tr);       /* mlklhd.c  */
void aproxlkl(Tree *tr);
int isacid(char c);           /* models.c  */
int acid2int(char c);
void rtfdata(double a[TPMRADIX][TPMRADIX], double *f);


/******************************************************************************/
/* getseq.c  reading sequence data from file (PHYLIP sequential format)       */
/******************************************************************************/


/***************************** internal functions *****************************/


/* read first ten characters of current line as identifier */
void readid(FILE *infp, int t)
{
	int i, flag;


	for (i = 0; i < 10; i++) {
		Identif[t][i] = fgetc(infp);
		if (ferror(infp) || Identif[t][i] == '\n' || Identif[t][i] == '\t') {
			fprintf(stderr, "\nERROR: I can't read the identifier");
			fprintf(stderr, " (taxum # %d)\n", t + 1);
			exit(1);
		}
	}
	
	/* convert blanks in species name to '_' */
	flag = FALSE;
	for (i = 9; i > -1; i--) {
		if (flag == FALSE) {
			if (Identif[t][i] != ' ') flag = TRUE; 
		} else {
			if (Identif[t][i] == ' ') Identif[t][i] = '_';
		}
	}
}


/* read sequences of one data set */
void getseqs(FILE *ifp)
{
	int notu, sn;
	char c;
	
	
	for (notu = 0; notu < Maxspc; notu++) {
		readid(ifp, notu);
		for (sn = 0; sn < Maxsite; sn++) {
			if (fscanf(ifp, "%c", &c) != 1) {
			fprintf(stderr, "\nERROR: I can't read character");
				fprintf(stderr, " (taxum # %d, ", notu + 1);
				fprintf(stderr, "character # %d)\n", sn + 1);
				exit(1);
			}
			
			if (isacid(c) == FALSE) {
				fprintf(stderr, "\nERROR: Invalid character ");
				fprintf(stderr, "'%c'", c);
				fprintf(stderr, " (taxum # %d, character # %d)\n",
					notu + 1, sn + 1);
				exit(1);
			}
			
			Seqchar[notu][sn] = acid2int(c);
		}
		
		/* make sure that we get to the next line */
		do{
			c = fgetc(ifp);
			if (ferror(ifp)) {
				fprintf(stderr, "\nERROR: I can't find the next line");
				fprintf(stderr, " (taxum # %d)\n", notu + 1);
				exit(1);
			}
		} while (c != '\n');
	}
}


/* initialize identifer array */
void initid(int t)
{
	int i, j;


	Identif = new_cmatrix(t, 10);
	
	for (i = 0; i < t; i++)
		for (j = 0; j < 10; j++)
			Identif[i][j] = ' ';
}


/***************************** exported functions *****************************/


/* read first line of sequence data set */
void getsizesites(FILE *ifp)
{
	char c;

	
	if (fscanf(ifp, "%d", &Maxspc) != 1) {
			fprintf(stderr, "\nERROR: I can't read the number of taxa\n");
			exit(1);
	}
	
	if (fscanf(ifp, "%d", &Maxsite) != 1) {
			fprintf(stderr, "\nERROR: I can't read the number of sites\n");
			exit(1);
	}
	
	/* make sure that we get to the next line */
	do{
		c = fgetc(ifp);
		if (ferror(ifp)) {
			fprintf(stderr, "\nERROR: I can't find the second line\n");
			exit(1);
		}
	} while (c != '\n');
}


/* read one data set */
void getdataset(FILE *ifp)
{
	initid(Maxspc);
	Seqchar = new_cmatrix(Maxspc, Maxsite);
	getseqs(ifp);
}


/* print identifier of specified taxum in full 10 char length */
void fputid10(FILE *ofp, int t)
{	
	int i;


	for (i = 0; i < 10; i++) fputc(Identif[t][i], ofp);
}


/* print identifier of specified taxum up to first space */
int fputid(FILE *ofp, int t)
{	
	int i;
	
	
	i = 0;
	while (Identif[t][i] != ' ' && i < 10) {
		fputc(Identif[t][i], ofp);
		i++;
	}
	
	return i;
}


/******************************************************************************/
/* seqproc.c  reorganisation of sequence data information                     */
/******************************************************************************/


/***************************** internal functions *****************************/


void convseq(imatrix seqconint, int maxspc, int numptrn)
{
	int i, j;


	for (i = 0; i < maxspc; i++) {
		for (j = 0; j < numptrn; j++) {
			if (seqconint[i][j] == TPMRADIX) {
				seqconint[i][j] = -1;
			}
		}
	}
}


void getfreqepm(cmatrix seqchar, double *freqemp, int maxspc, int maxsite)
{
	int i, j, all;
	ivector gene;


	gene = new_ivector(TPMRADIX + 1);
	for (i = 0; i < TPMRADIX + 1; i++)
		gene[i] = 0;
	for (i = 0; i < maxspc; i++) {
		for (j = 0; j < maxsite; j++) {
			gene[(int)seqchar[i][j]]++;
		}
	}
	all = maxspc * maxsite - gene[TPMRADIX];
	for (i = 0; i < TPMRADIX; i++) {
		freqemp[i] = (double)gene[i] / (double) all;
		
	}
	free_ivector(gene);
}


void convfreq(double *freqemp)
{
	int i, maxi;
	double freq, maxfreq, sum;


	sum = 0.0;
	maxfreq = 0.0;
	for (i = 0; i < TPMRADIX; i++) {
		freq = freqemp[i];
		if (freq < MINFREQ)
			freqemp[i] = MINFREQ;
		if (freq > maxfreq) {
			maxfreq = freq;
			maxi = i;
		}
		sum += freqemp[i];
	}
	freqemp[maxi] += 1.0 - sum;
}


void radixsort(cmatrix seqchar, ivector alias, int maxspc, int maxsite,
	int *numptrn)
{
	int i, j, k, l, n, pass;
	int *awork;
	int *count;


	awork = new_ivector(maxsite);
	count = new_ivector(TPMRADIX+1);
	for (i = 0; i < maxsite; i++)
		alias[i] = i;
	for (pass = maxspc - 1; pass >= 0; pass--) {
		for (j = 0; j < TPMRADIX+1; j++)
			count[j] = 0;
		for (i = 0; i < maxsite; i++)
			count[seqchar[pass][alias[i]]]++;
		for (j = 1; j < TPMRADIX+1; j++)
			count[j] += count[j-1];
		for (i = maxsite-1; i >= 0; i--)
			awork[ --count[seqchar[pass][alias[i]]] ] = alias[i];
		for (i = 0; i < maxsite; i++)
			alias[i] = awork[i];
	}
	free_ivector(awork);
	free_ivector(count);
	n = 1;
	for (j = 1; j < maxsite; j++) {
		k = alias[j];
		l = alias[j-1];
		for (i = 0; i < maxspc; i++) {
			if (seqchar[i][l] != seqchar[i][k]) {
				n++;
				break;
			}
		}
	}
	*numptrn = n;
}


void condenceseq(cmatrix seqchar, ivector alias, imatrix seqconint,
	ivector weight, int maxspc, int maxsite, int numptrn)
{
	int i, j, k, n;
	int agree_flag; /* boolean */


	n = 0;
	k = alias[n];
	for (i = 0; i < maxspc; i++) {
		seqconint[i][n] = seqchar[i][k];
	}
	weight[n] = 1;
	for (j = 1; j < maxsite; j++) {
		k = alias[j];
		agree_flag = TRUE;
		for (i = 0; i < maxspc; i++) {
			if (seqconint[i][n] != seqchar[i][k]) {
				agree_flag = FALSE;
				break;
			}
		}
		if (agree_flag == FALSE) {
			n++;
			for (i = 0; i < maxspc; i++) {
				seqconint[i][n] = seqchar[i][k];
			}
			weight[n] = 1;
		} else {
			weight[n]++;
		}
	}
	n++;
	if (numptrn != n) {
		fprintf(stderr,
			"\nERROR: I have a problem in condenceseq [numptrn != n]\n");
		exit(1);
	}
}


/***************************** exported functions *****************************/


void evaluateseqs()
{	
	ivector alias;


	getfreqepm(Seqchar, Freqemp, Maxspc, Maxsite);
	alias = new_ivector(Maxsite);
	radixsort(Seqchar, alias, Maxspc, Maxsite, &Numptrn);
	Seqconint = new_imatrix(Maxspc, Numptrn);
	Weight = new_ivector(Numptrn);
	condenceseq(Seqchar, alias, Seqconint, Weight, Maxspc, Maxsite, Numptrn);
	convseq(Seqconint, Maxspc, Numptrn);
	free_ivector(alias);
	if (Frequ_optn) convfreq(Freqemp);
}


/******************************************************************************/
/* distan.c  estimation of distance matrix from sequence data                 */
/******************************************************************************/


/***************************** internal functions *****************************/


static void distan(double *dis, ivector seqi, ivector seqj,
	ivector seqw, int nsite)
{
	int i, j, k, l;
	int maxloop = 10;
	double dist, distold, distdiff;
	double lkl, ld1, lkld1, lkld2;
	dmattpmty tprob, tdiff1, tdiff2;


	dist = *dis;
	for (l = 0; l < maxloop; l++) {
		tdiffmtrx(dist, tprob, tdiff1, tdiff2);
		lkld1 = 0.0;
		lkld2 = 0.0;
		for (k = 0; k < nsite; k++) {
			i = seqi[k];
			j = seqj[k];
			lkl = tprob[i][j];
			ld1 = tdiff1[i][j] / lkl;
			lkld1 += ld1 * (double) seqw[k];
			lkld2 += (tdiff2[i][j] / lkl - ld1 * ld1 ) * (double) seqw[k];
		}
		distold = dist;
		distdiff = -(lkld1 / lkld2);
		if (distdiff > MAXARC - distold) {
			if (distold < MAXARC / 2.0)
				dist = MINARC;
			else
				dist = MAXARC;
		} else
			dist += distdiff;
		if (dist < MINARC) dist = MINARC;
		if (dist > MAXARC) dist = MAXARC;
		if (fabs(distold - dist) < DEPSILON)  /* EPSILON */
			break;
	}
	*dis = dist;
}

void distance(dmatrix distanmat, cmatrix seqchar, int maxspc, int numsite)
{
	int i, j, k, l, nsite, diff;
	double dist;
	ivector seqi, seqj, seqw;
	cvector seqchi, seqchj;
	int gene[TPMRADIX + 1];


	seqi = new_ivector(numsite);
	seqj = new_ivector(numsite);
	seqw = new_ivector(numsite);
	for (i = 0; i < maxspc-1; i++) {
		seqchi = seqchar[i];
		for (j = i+1; j < maxspc; j++) {
			seqchj = seqchar[j];
			for ( k = 0; k < TPMRADIX + 1; k++) gene[k] = 0;
			for ( k = 0; k < numsite; k++) {
				if (seqchi[k] == seqchj[k]) {
					gene[seqchi[k]]++;
					seqw[k] = FALSE;
				} else if (seqchi[k] == TPMRADIX || seqchj[k] == TPMRADIX) {
					seqw[k] = FALSE;
				} else {
					seqw[k] = TRUE;
				}
			}
			for ( k = 0, l = 0; k < numsite; k++) {
				if (seqw[k]) {
					seqi[l] = seqchi[k];
					seqj[l] = seqchj[k];
					seqw[l] = 1;
					l++;
				}
			}
			diff = l;
			for ( k = 0; k < TPMRADIX; k++) {
				if (gene[k]) {
					seqi[l] = k;
					seqj[l] = k;
					seqw[l] = gene[k];
					l++;
				}
			}
			nsite = l;
			if (diff == 0)
				dist = -log(1.0) * 100.0;
			else if (diff < numsite)
				dist = -log(1.0 - (double) diff/(double) numsite) * 100.0;
			else
				dist = -log(1.0 - (double) diff/(double) (numsite+1.0)) * 100.0;
			if (dist < MINARC) dist = MINARC;
			if (dist > MAXARC) dist = MAXARC;
			distan(&dist, seqi, seqj, seqw, nsite);
			distanmat[i][j] = dist;
			distanmat[j][i] = dist;
		}
		distanmat[i][i] = 0.0;
	}
	distanmat[maxspc-1][maxspc-1] = 0.0;
	free_ivector(seqw);
	free_ivector(seqj);
	free_ivector(seqi);
}


/***************************** exported functions *****************************/


/* compute distance matrix */
void computedistan()
{
	Distanmat = new_dmatrix(Maxspc, Maxspc);
	distance(Distanmat, Seqchar, Maxspc, Maxsite);
}


/******************************************************************************/
/* tranprob.c  computation of Pij(t) from RTF                                 */
/******************************************************************************/


/***************************** internal functions *****************************/


void elmhes(dmattpmty a, int *ordr, int n)
{
	int m, j, i;
	double y, x;


	for (i = 0; i < n; i++)
		ordr[i] = 0;
	for (m = 2; m < n; m++) {
		x = 0.0;
		i = m;
		for (j = m; j <= n; j++) {
			if (fabs(a[j - 1][m - 2]) > fabs(x)) {
				x = a[j - 1][m - 2];
				i = j;
			}
		}
		ordr[m - 1] = i;      /* vector */
		if (i != m) {
			for (j = m - 2; j < n; j++) {
				y = a[i - 1][j];
				a[i - 1][j] = a[m - 1][j];
				a[m - 1][j] = y;
			}
			for (j = 0; j < n; j++) {
				y = a[j][i - 1];
				a[j][i - 1] = a[j][m - 1];
				a[j][m - 1] = y;
			}
		}
		if (x != 0.0) {
			for (i = m; i < n; i++) {
				y = a[i][m - 2];
				if (y != 0.0) {
					y /= x;
					a[i][m - 2] = y;
					for (j = m - 1; j < n; j++)
						a[i][j] -= y * a[m - 1][j];
					for (j = 0; j < n; j++)
						a[j][m - 1] += y * a[j][i];
				}
			}
		}
	}
}


void eltran(dmattpmty a, dmattpmty zz, int *ordr, int n)
{
	int i, j, m;


	for (i = 0; i < n; i++) {
		for (j = i + 1; j < n; j++) {
			zz[i][j] = 0.0;
			zz[j][i] = 0.0;
		}
		zz[i][i] = 1.0;
	}
	if (n <= 2)
		return;
	for (m = n - 1; m >= 2; m--) {
		for (i = m; i < n; i++)
			zz[i][m - 1] = a[i][m - 2];
		i = ordr[m - 1];
		if (i != m) {
			for (j = m - 1; j < n; j++) {
				zz[m - 1][j] = zz[i - 1][j];
				zz[i - 1][j] = 0.0;
			}
			zz[i - 1][m - 1] = 1.0;
		}
	}
}


void cdiv(double ar, double ai, double br, double bi,
	double *cr, double *ci)
{
	double s, ars, ais, brs, bis;


	s = fabs(br) + fabs(bi);
	ars = ar / s;
	ais = ai / s;
	brs = br / s;
	bis = bi / s;
	s = brs * brs + bis * bis;
	*cr = (ars * brs + ais * bis) / s;
	*ci = (ais * brs - ars * bis) / s;
}


void hqr2(int n, int low, int hgh, int *err, dmattpmty h, dmattpmty zz,
	double *wr, double *wi)
{
	int i, j, k, l, m, en, na, itn, its;
	double p, q, r, s, t, w, x, y, ra, sa, vi, vr, z, norm, tst1, tst2;
	int notlas; /* boolean */


	*err = 0;
	norm = 0.0;
	k = 1;
	/* store roots isolated by balanc and compute matrix norm */
	for (i = 0; i < n; i++) {
		for (j = k - 1; j < n; j++)
			norm += fabs(h[i][j]);
		k = i + 1;
		if (i + 1 < low || i + 1 > hgh) {
			wr[i] = h[i][i];
			wi[i] = 0.0;
		}
	}
	en = hgh;
	t = 0.0;
	itn = n * 30;
	while (en >= low) {	/* search for next eigenvalues */
		its = 0;
		na = en - 1;
		while (en >= 1) {
			/* look for single small sub-diagonal element */
			for (l = en; l > low; l--) {
				s = fabs(h[l - 2][l - 2]) + fabs(h[l - 1][l - 1]);
				if (s == 0.0)
					s = norm;
				tst1 = s;
				tst2 = tst1 + fabs(h[l - 1][l - 2]);
				if (tst2 == tst1)
					goto L100;
			}
			l = low;
	L100:
			x = h[en - 1][en - 1];	/* form shift */
			if (l == en || l == na)
				break;
			if (itn == 0) { /* all eigenvalues have not converged */
				*err = en;
				goto Lerror;
			}
			y = h[na - 1][na - 1];
			w = h[en - 1][na - 1] * h[na - 1][en - 1];
			/* form exceptional shift */
			if (its == 10 || its == 20) {
				t += x;
				for (i = low - 1; i < en; i++)
					h[i][i] -= x;
				s = fabs(h[en - 1][na - 1]) + fabs(h[na - 1][en - 3]);
				x = 0.75 * s;
				y = x;
				w = -0.4375 * s * s;
			}
			its++;
			itn--;
			/* look for two consecutive small sub-diagonal elements */
			for (m = en - 2; m >= l; m--) {
				z = h[m - 1][m - 1];
				r = x - z;
				s = y - z;
				p = (r * s - w) / h[m][m - 1] + h[m - 1][m];
				q = h[m][m] - z - r - s;
				r = h[m + 1][m];
				s = fabs(p) + fabs(q) + fabs(r);
				p /= s;
				q /= s;
				r /= s;
				if (m == l)
					break;
				tst1 = fabs(p) *
						(fabs(h[m - 2][m - 2]) + fabs(z) + fabs(h[m][m]));
				tst2 = tst1 + fabs(h[m - 1][m - 2]) * (fabs(q) + fabs(r));
				if (tst2 == tst1)
					break;
			}
			for (i = m + 2; i <= en; i++) {
				h[i - 1][i - 3] = 0.0;
				if (i != m + 2)
					h[i - 1][i - 4] = 0.0;
			}
			for (k = m; k <= na; k++) {
				notlas = (k != na);
				if (k != m) {
					p = h[k - 1][k - 2];
					q = h[k][k - 2];
					r = 0.0;
					if (notlas)
						r = h[k + 1][k - 2];
					x = fabs(p) + fabs(q) + fabs(r);
					if (x != 0.0) {
						p /= x;
						q /= x;
						r /= x;
					}
				}
				if (x != 0.0) {
					if (p < 0.0) /* sign */
						s = - sqrt(p * p + q * q + r * r);
					else
						s = sqrt(p * p + q * q + r * r);
					if (k != m)
						h[k - 1][k - 2] = -s * x;
					else {
						if (l != m)
							h[k - 1][k - 2] = -h[k - 1][k - 2];
					}
					p += s;
					x = p / s;
					y = q / s;
					z = r / s;
					q /= p;
					r /= p;
					if (!notlas) {
						for (j = k - 1; j < n; j++) {	/* row modification */
							p = h[k - 1][j] + q * h[k][j];
							h[k - 1][j] -= p * x;
							h[k][j] -= p * y;
						}
						j = (en < (k + 3)) ? en : (k + 3); /* min */
						for (i = 0; i < j; i++) {	/* column modification */
							p = x * h[i][k - 1] + y * h[i][k];
							h[i][k - 1] -= p;
							h[i][k] -= p * q;
						}
						/* accumulate transformations */
						for (i = low - 1; i < hgh; i++) {
							p = x * zz[i][k - 1] + y * zz[i][k];
							zz[i][k - 1] -= p;
							zz[i][k] -= p * q;
						}
					} else {
						for (j = k - 1; j < n; j++) {	/* row modification */
							p = h[k - 1][j] + q * h[k][j] + r * h[k + 1][j];
							h[k - 1][j] -= p * x;
							h[k][j] -= p * y;
							h[k + 1][j] -= p * z;
						}
						j = (en < (k + 3)) ? en : (k + 3); /* min */
						for (i = 0; i < j; i++) {	/* column modification */
							p = x * h[i][k - 1] + y * h[i][k] + z * h[i][k + 1];
							h[i][k - 1] -= p;
							h[i][k] -= p * q;
							h[i][k + 1] -= p * r;
						}
						/* accumulate transformations */
						for (i = low - 1; i < hgh; i++) {
							p = x * zz[i][k - 1] + y * zz[i][k] +
								z * zz[i][k + 1];
							zz[i][k - 1] -= p;
							zz[i][k] -= p * q;
							zz[i][k + 1] -= p * r;
						}
					}
				}
			}	       /* for k */
		}		       /* while infinite loop */
		if (l == en) {	       /* one root found */
			h[en - 1][en - 1] = x + t;
			wr[en - 1] = h[en - 1][en - 1];
			wi[en - 1] = 0.0;
			en = na;
			continue;
		}
		y = h[na - 1][na - 1];
		w = h[en - 1][na - 1] * h[na - 1][en - 1];
		p = (y - x) / 2.0;
		q = p * p + w;
		z = sqrt(fabs(q));
		h[en - 1][en - 1] = x + t;
		x = h[en - 1][en - 1];
		h[na - 1][na - 1] = y + t;
		if (q >= 0.0) {	       /* real pair */
			if (p < 0.0) /* sign */
				z = p - fabs(z);
			else
				z = p + fabs(z);
			wr[na - 1] = x + z;
			wr[en - 1] = wr[na - 1];
			if (z != 0.0)
				wr[en - 1] = x - w / z;
			wi[na - 1] = 0.0;
			wi[en - 1] = 0.0;
			x = h[en - 1][na - 1];
			s = fabs(x) + fabs(z);
			p = x / s;
			q = z / s;
			r = sqrt(p * p + q * q);
			p /= r;
			q /= r;
			for (j = na - 1; j < n; j++) {	/* row modification */
				z = h[na - 1][j];
				h[na - 1][j] = q * z + p * h[en - 1][j];
				h[en - 1][j] = q * h[en - 1][j] - p * z;
			}
			for (i = 0; i < en; i++) {	/* column modification */
				z = h[i][na - 1];
				h[i][na - 1] = q * z + p * h[i][en - 1];
				h[i][en - 1] = q * h[i][en - 1] - p * z;
			}
			/* accumulate transformations */
			for (i = low - 1; i < hgh; i++) {
				z = zz[i][na - 1];
				zz[i][na - 1] = q * z + p * zz[i][en - 1];
				zz[i][en - 1] = q * zz[i][en - 1] - p * z;
			}
		} else {	       /* complex pair */
			wr[na - 1] = x + p;
			wr[en - 1] = x + p;
			wi[na - 1] = z;
			wi[en - 1] = -z;
		}
		en -= 2;
	}			       /* while en >= low */
	/* backsubstitute to find vectors of upper triangular form */
	if (norm != 0.0) {
		for (en = n; en >= 1; en--) {
			p = wr[en - 1];
			q = wi[en - 1];
			na = en - 1;
			if (q == 0.0) {/* real vector */
				m = en;
				h[en - 1][en - 1] = 1.0;
				if (na != 0) {
					for (i = en - 2; i >= 0; i--) {
						w = h[i][i] - p;
						r = 0.0;
						for (j = m - 1; j < en; j++)
							r += h[i][j] * h[j][en - 1];
						if (wi[i] < 0.0) {
							z = w;
							s = r;
						} else {
							m = i + 1;
							if (wi[i] == 0.0) {
								t = w;
								if (t == 0.0) {
									tst1 = norm;
									t = tst1;
									do {
										t = 0.01 * t;
										tst2 = norm + t;
									} while (tst2 > tst1);
								}
								h[i][en - 1] = -(r / t);
							} else {	/* solve real equations */
								x = h[i][i + 1];
								y = h[i + 1][i];
								q = (wr[i] - p) * (wr[i] - p) + wi[i] * wi[i];
								t = (x * s - z * r) / q;
								h[i][en - 1] = t;
								if (fabs(x) > fabs(z))
									h[i + 1][en - 1] = (-r - w * t) / x;
								else
									h[i + 1][en - 1] = (-s - y * t) / z;
							}
							/* overflow control */
							t = fabs(h[i][en - 1]);
							if (t != 0.0) {
								tst1 = t;
								tst2 = tst1 + 1.0 / tst1;
								if (tst2 <= tst1) {
									for (j = i; j < en; j++)
										h[j][en - 1] /= t;
								}
							}
						}
					}
				}
			} else if (q > 0.0) {
				m = na;
				if (fabs(h[en - 1][na - 1]) > fabs(h[na - 1][en - 1])) {
					h[na - 1][na - 1] = q / h[en - 1][na - 1];
					h[na - 1][en - 1] = (p - h[en - 1][en - 1]) /
														h[en - 1][na - 1];
				} else
					cdiv(0.0, -h[na - 1][en - 1], h[na - 1][na - 1] - p, q,
						&h[na - 1][na - 1], &h[na - 1][en - 1]);
				h[en - 1][na - 1] = 0.0;
				h[en - 1][en - 1] = 1.0;
				if (en != 2) {
					for (i = en - 3; i >= 0; i--) {
						w = h[i][i] - p;
						ra = 0.0;
						sa = 0.0;
						for (j = m - 1; j < en; j++) {
							ra += h[i][j] * h[j][na - 1];
							sa += h[i][j] * h[j][en - 1];
						}
						if (wi[i] < 0.0) {
							z = w;
							r = ra;
							s = sa;
						} else {
							m = i + 1;
							if (wi[i] == 0.0)
								cdiv(-ra, -sa, w, q, &h[i][na - 1],
									&h[i][en - 1]);
							else {	/* solve complex equations */
								x = h[i][i + 1];
								y = h[i + 1][i];
								vr = (wr[i] - p) * (wr[i] - p) + wi[i] * wi[i] -
										q * q;
								vi = (wr[i] - p) * 2.0 * q;
								if (vr == 0.0 && vi == 0.0) {
									tst1 = norm * (fabs(w) + fabs(q) + fabs(x) +
												   fabs(y) + fabs(z));
									vr = tst1;
									do {
										vr = 0.01 * vr;
										tst2 = tst1 + vr;
									} while (tst2 > tst1);
								}
								cdiv(x * r - z * ra + q * sa,
									 x * s - z * sa - q * ra, vr, vi,
								     &h[i][na - 1], &h[i][en - 1]);
								if (fabs(x) > fabs(z) + fabs(q)) {
									h[i + 1]
										[na - 1] = (q * h[i][en - 1] -
													w * h[i][na - 1] - ra) / x;
									h[i + 1][en - 1] = (-sa - w * h[i][en - 1] -
														q * h[i][na - 1]) / x;
								} else
									cdiv(-r - y * h[i][na - 1],
										 -s - y * h[i][en - 1], z, q,
									     &h[i + 1][na - 1], &h[i + 1][en - 1]);
							}
							/* overflow control */
							t = (fabs(h[i][na - 1]) > fabs(h[i][en - 1])) ?
								 fabs(h[i][na - 1]) : fabs(h[i][en - 1]);
							if (t != 0.0) {
								tst1 = t;
								tst2 = tst1 + 1.0 / tst1;
								if (tst2 <= tst1) {
									for (j = i; j < en; j++) {
										h[j][na - 1] /= t;
										h[j][en - 1] /= t;
									}
								}
							}
						}
					}
				}
			}
		}
		/* end back substitution. vectors of isolated roots */
		for (i = 0; i < n; i++) {
			if (i + 1 < low || i + 1 > hgh) {
				for (j = i; j < n; j++)
					zz[i][j] = h[i][j];
			}
		}
		/* multiply by transformation matrix to give vectors of
		 * original full matrix. */
		for (j = n - 1; j >= low - 1; j--) {
			m = ((j + 1) < hgh) ? (j + 1) : hgh; /* min */
			for (i = low - 1; i < hgh; i++) {
				z = 0.0;
				for (k = low - 1; k < m; k++)
					z += zz[i][k] * h[k][j];
				zz[i][j] = z;
			}
		}
	}
	return;

Lerror: /* two roots found and complex vector */
	fprintf(stderr, "\nERROR: Two roots found or complex vector [hqr2]\n");
	exit(1);
}


void tpmonepam(dmattpmty a, dvectpmty forg, dvectpmty femp, dvectpmty ftpm)
{
	int i, j;
	double delta, temp, sum;
	dvectpmty m;


	if (Frequ_optn) {
		for (i = 0; i < TPMRADIX; i++) {
			for (j = 0; j < TPMRADIX; j++)
				a[i][j] *= (femp[i]/forg[i]) * (femp[j]/forg[j]);
		}
		for (i = 0; i < TPMRADIX; i++) ftpm[i] = femp[i];
	} else {
		for (i = 0; i < TPMRADIX; i++) ftpm[i] = forg[i];
	}
	for (i = 0, sum = 0.0; i < TPMRADIX; i++) {
		for (j = 0, temp = 0.0; j < TPMRADIX; j++) temp += a[i][j];
		m[i] = temp / ftpm[i];
		sum += temp;
	
	}
	delta = 0.01 / sum; /* 1 PAM */
	for (i = 0; i < TPMRADIX; i++) {
		for (j = 0; j < TPMRADIX; j++) {
			if (i != j)
				a[i][j] = delta * a[i][j] / ftpm[i]; /* * m[i] */
			else
				a[i][j] = 1.0 - delta * m[i];
		}
	}
}


void eigensystem(double *eval, dmattpmty evec, double *femp, double *ftpm)
{
	dmattpmty a, b;
	dvectpmty evali, forg;
	ivectpmty ordr;
	int i, j, k, err;
	double zero;
	int error; /* boolean */


	rtfdata(a, forg); /* get RTF matrix and frequencies -> model.c */
	for (i = 0; i < TPMRADIX; i++) { /* is RTF (in variable a) symmetric ? */
		for (j = 0; j < TPMRADIX; j++) {
			if (a[i][j] != a[j][i]) {
				fprintf(stderr,
					"\nERROR: RTF matrix is not symmetric [eigensystem]\n");
				exit(1);
			}
		}
	}
	if (Frequ_optn) {
		for (i = 0; i < TPMRADIX - 1; i++) {
			for (j = i + 1; j < TPMRADIX; j++) {
				if (femp[i] == femp[j]) {
					femp[i] += 0.00001;
					femp[j] -= 0.00001;
				}
			}
		}
	}
	tpmonepam(a, forg, femp, ftpm); /* make 1 PAM TPM */
	for (i = 0; i < TPMRADIX; i++) { /* copy a to b */
		for (j = 0; j < TPMRADIX; j++)
			b[i][j] = a[i][j];
	}
	error = FALSE;
	elmhes(a, ordr, TPMRADIX);
	eltran(a, evec, ordr, TPMRADIX);
	hqr2(TPMRADIX, 1, TPMRADIX, &err, a, evec, eval, evali);
	for (j = 0; j < TPMRADIX; j++) {
		for (i = 0, zero = 0.0; i < TPMRADIX; i++) {
			for (k = 0; k < TPMRADIX; k++) zero += b[i][k] * evec[k][j];
			zero -= eval[j] * evec[i][j];
			if (fabs(zero) > 1.0e-5) error = TRUE;
		}
	}
	if (error) {
		fprintf(stderr,
			"\nERROR: I have a problem with the TPM eigensystem\n");
		exit(1);
	}
}


void luinverse(dmattpmty omtrx, dmattpmty imtrx, int size)
{
    double eps = 1.0e-20; /* ! */
	int i, j, k, l, maxi, idx, ix, jx;
	double sum, tmp, maxb, aw;
	ivectpmty index;
	double *wk;


	wk = (double *) malloc((unsigned)size * sizeof(double));
	aw = 1.0;
	for (i = 0; i < size; i++) {
		maxb = 0.0;
		for (j = 0; j < size; j++) {
			if (fabs(omtrx[i][j]) > maxb)
				maxb = fabs(omtrx[i][j]);
		}
		if (maxb == 0.0) {
			fprintf(stderr,
				"\nERROR: Singular matrix [luinverse]\n");
			exit(1);
		}
		wk[i] = 1.0 / maxb;
	}
	for (j = 0; j < size; j++) {
		for (i = 0; i < j; i++) {
			sum = omtrx[i][j];
			for (k = 0; k < i; k++)
				sum -= omtrx[i][k] * omtrx[k][j];
			omtrx[i][j] = sum;
		}
		maxb = 0.0;
		for (i = j; i < size; i++) {
			sum = omtrx[i][j];
			for (k = 0; k < j; k++)
				sum -= omtrx[i][k] * omtrx[k][j];
			omtrx[i][j] = sum;
			tmp = wk[i] * fabs(sum);
			if (tmp >= maxb) {
				maxb = tmp;
				maxi = i;
			}
		}
		if (j != maxi) {
			for (k = 0; k < size; k++) {
				tmp = omtrx[maxi][k];
				omtrx[maxi][k] = omtrx[j][k];
				omtrx[j][k] = tmp;
			}
			aw = -aw;
			wk[maxi] = wk[j];
		}
		index[j] = maxi;
		if (omtrx[j][j] == 0.0)
			omtrx[j][j] = eps;
		if (j != size - 1) {
			tmp = 1.0 / omtrx[j][j];
			for (i = j + 1; i < size; i++)
				omtrx[i][j] *= tmp;
		}
	}
	for (jx = 0; jx < size; jx++) {
		for (ix = 0; ix < size; ix++)
			wk[ix] = 0.0;
		wk[jx] = 1.0;
		l = -1;
		for (i = 0; i < size; i++) {
			idx = index[i];
			sum = wk[idx];
			wk[idx] = wk[i];
			if (l != -1) {
				for (j = l; j < i; j++)
					sum -= omtrx[i][j] * wk[j];
			} else if (sum != 0.0)
				l = i;
			wk[i] = sum;
		}
		for (i = size - 1; i >= 0; i--) {
			sum = wk[i];
			for (j = i + 1; j < size; j++)
				sum -= omtrx[i][j] * wk[j];
			wk[i] = sum / omtrx[i][i];
		}
		for (ix = 0; ix < size; ix++)
			imtrx[ix][jx] = wk[ix];
	}
	free((char *)wk);
	wk = NULL;
}


void mproduct(dmattpmty am, dmattpmty bm, dmattpmty cm, int na, int nb, int nc)
{
	int ia, ib, ic;
	double sum;


	for (ia = 0; ia < na; ia++) {
		for (ic = 0; ic < nc; ic++) {
			sum = 0.0;
			for (ib = 0; ib < nb; ib++) sum += am[ia][ib] * bm[ib][ic];
			cm[ia][ic] = sum;
		}
	}
}


void checkevector(dmattpmty imtrx, int nn)
{
	int i, j;


	for (i = 0; i < nn; i++) {
		for (j = 0; j < nn; j++) {
			if (i == j) {
				if (fabs(imtrx[i][j] - 1.0) > 1.0e-5) {
					fprintf(stderr, "\nERROR: Eigenvector [checkevector 1]\n");
					exit(1);
				}
			} else {
				if (fabs(imtrx[i][j]) > 1.0e-5) {
					fprintf(stderr, "\nERROR: Eigenvector [checkevector 2]\n");
					exit(1);
				}
			}
		}
	}
}


/***************************** exported functions *****************************/


/* make transition probability matrix */
void tranprobmat()
{
	dmattpmty matx;
	int i, j;
	double temp;


	eigensystem(Eval, Evec, Freqemp, Freqtpm);
	for (i = 0; i < TPMRADIX; i++) {
		for (j = 0; j < TPMRADIX; j++)
			matx[i][j] = Evec[i][j];
	}
	luinverse(matx, Ievc, TPMRADIX);
	mproduct(Evec, Ievc, matx, TPMRADIX, TPMRADIX, TPMRADIX);
	checkevector(matx, TPMRADIX);
	for (i = 0; i < TPMRADIX; i++) {
		temp = log(Eval[i]);
		Eval[i] = temp;
		Evl2[i] = temp * temp;
	}
}


void tprobmtrx(double arc, dmattpmty tpr)
{
	/* TRANSITION PROBABILITY MATRIX */
	register int i, j, k;
	register double temp;
	dmattpmty iexp;


	for (k = 0; k < TPMRADIX; k++) {
		temp = exp(arc * Eval[k]);
		for (j = 0; j < TPMRADIX; j++)
			iexp[k][j] = Ievc[k][j] * temp;
	}
	for (i = 0; i < TPMRADIX; i++) {
		for (j = 0; j < TPMRADIX; j++) {
			temp = 0.0;
			for (k = 0; k < TPMRADIX; k++)
				temp += Evec[i][k] * iexp[k][j];
			tpr[i][j] = temp;
		}
	}
}


/* transition probability and difference matrix */
void tdiffmtrx(double arc, dmattpmty tpr, dmattpmty td1, dmattpmty td2)
{
	register int i, j, k;
	register double x, tp, t1, t2;
	dmattpmty iexp;
	dvector evc;


	for (k = 0; k < TPMRADIX; k++) {
		x = exp(arc * Eval[k]);
		for (j = 0; j < TPMRADIX; j++)
			iexp[k][j] = Ievc[k][j] * x;
	}
	for (i = 0; i < TPMRADIX; i++) {
		evc = Evec[i];
		for (j = 0; j < TPMRADIX; j++) {
			tp = t1 = t2 = 0.0;
			for (k = 0; k < TPMRADIX; k++) {
				x = evc[k] * iexp[k][j];
				tp += x;
				t1 += x * Eval[k];
				t2 += x * Evl2[k];
			}
			tpr[i][j] = tp;
			td1[i][j] = t1;
			td2[i][j] = t2;
		}
	}
}


/******************************************************************************/
/* mlklhd.c  computation of maximum likelihood                                */
/******************************************************************************/


/***************************** internal functions *****************************/


void prodpart(Node *op)
{
	Node *cp;
	int i, j;
	dvector opb, cpb;


	cp = op;
	while (cp->isop->isop != op) {
		cp = cp->isop;
		opb = *(op->iprob);
		cpb = *(cp->iprob);
		for (i = 0; i < Numptrn; i++)
			for (j = 0; j < TPMRADIX; j++) *opb++ *= *cpb++;
	}
}


void partilkl(Node *op)
{
	int i, j, k;
	double sum;
	dmattpmty tprob;
	dmatrix oprob, cprob;
	dvector opb, cpb, cpb2, tpb;


	tprobmtrx(op->length, tprob);
	oprob = op->iprob;
	cprob = op->kinp->isop->iprob;
	for (k = 0; k < Numptrn; k++) {
		opb = oprob[k];
		cpb = cprob[k];
		tpb = *tprob;
		for (i = 0; i < TPMRADIX; i++) {
			cpb2 = cpb;
			sum  = *tpb++ * *cpb2++;
			for (j = 0; j < TPMRADIX - 1; j++) sum += *tpb++ * *cpb2++;
			opb[i] = sum;
		}
	}
}


void partelkl(Node *op)
{
	int i, j, k;
	dmattpmty tprob;
	dmatrix oprob;
	ivector dseqi;
	dvector opb;


	tprobmtrx(op->length, tprob);
	oprob = op->iprob;
	dseqi = op->kinp->eprob;
	for (k = 0; k < Numptrn; k++) {
		opb = oprob[k];
		if ((j = dseqi[k]) >= 0) {
			for (i = 0; i < TPMRADIX; i++) opb[i] = tprob[i][j];
		} else {
			for (i = 0; i < TPMRADIX; i++) opb[i] = 1.0; /* !? */
		}
	}
}


void mlibranch(Node *op, int loop)
{
	int i, j, k, it, numloop;
	double sumlk, sumd1, sumd2, lkld1, lkld2, prod1, prod2, vari, lklhd;
	double arc, arcold, arcdiff, arcpre, slk, sd1, sd2;
	dmattpmty tprob, tdif1, tdif2;
	dmatrix oprob, cprob;
	dvector tpb, td1, td2, opb, cpb;


	oprob = op->isop->iprob;
	cprob = op->kinp->isop->iprob;
	arc = op->length;
	arcpre = arc;
	loop < 3 ? (numloop = 1) : (numloop = 3);
	for (it = 0; it < numloop; it++) {
		tdiffmtrx(arc, tprob, tdif1, tdif2);
		lkld1 = lkld2 = 0.0;
		for (k = 0; k < Numptrn; k++) {
			sumlk = sumd1 = sumd2 = 0.0;
			opb = oprob[k];
			cpb = cprob[k];
			for (i = 0; i < TPMRADIX; i++) {
				tpb = tprob[i];
				td1 = tdif1[i];
				td2 = tdif2[i];
				prod1 = Freqtpm[i] * opb[i];
				slk = sd1 = sd2 = 0.0;
				for (j = 0; j < TPMRADIX; j++) {
					prod2 = cpb[j];
					slk += prod2 * tpb[j];
					sd1 += prod2 * td1[j];
					sd2 += prod2 * td2[j];
				}
				sumlk += prod1 * slk;
				sumd1 += prod1 * sd1;
				sumd2 += prod1 * sd2;
			}
			sumd1 /= sumlk;
			lkld1 += sumd1 * Weight[k];
			lkld2 += (sumd2 / sumlk - sumd1 * sumd1) * Weight[k];
		}
		vari = 1.0 / fabs(lkld2);
		arcold = arc;
		arcdiff = - (lkld1 / lkld2);
		arc += arcdiff;
		if (arc > MIDARC && arcpre < 10.0) arc = MINARC;
		if (arc < MINARC) arc = MINARC;
		if (arc > MAXARC) arc = MAXARC;
		if (fabs(arcold - arc) < EPSILON) break;
	}
	op->kinp->length = arc;
	op->length = arc;
	op->lklhdl = vari;
	tprobmtrx(arc, tprob);
	if ((op->number == NUMIBRNCH-1 && fabs(arc-arcpre) < EPSILON) ||
		loop == MAXIT-1) {
		for (k = 0, lklhd = 0.0; k < Numptrn; k++) {
			opb = oprob[k];
			cpb = cprob[k];
			for (i = 0, sumlk = 0.0; i < TPMRADIX; i++) {
				tpb = tprob[i];
				prod1 = Freqtpm[i] * opb[i];

				for (j = 0, slk = 0.0; j < TPMRADIX; j++)
					slk += cpb[j] * tpb[j];
				sumlk += prod1 * slk;
			}
			sumlk = log(sumlk);	
			lklhd += sumlk * Weight[k];
		}
		Ctree->lklhd = lklhd;
	}
	oprob = op->iprob;
	for (k = 0; k < Numptrn; k++) {
		opb = oprob[k];
		cpb = cprob[k];
		for (i = 0; i < TPMRADIX; i++) {
			tpb = tprob[i];
			for (j = 0, sumlk = 0.0; j < TPMRADIX; j++)
				sumlk += tpb[j] * cpb[j];
			opb[i] = sumlk;
		}
	}
}


void mlebranch(Node *op, int loop)
{
	int i, j, k, it, numloop;
	double sumlk, sumd1, sumd2, lkld1, lkld2, prod, vari;
	double arc, arcold, arcdiff, arcpre;
	dmattpmty tprob, tdif1, tdif2;
	dmatrix oprob;
	ivector dseqi;
	dvector opb;


	oprob = op->isop->iprob;
	dseqi = op->kinp->eprob;
	arc = op->length;
	arcpre = arc;
	loop < 3 ? (numloop = 1) : (numloop = 3);
	for (it = 0; it < numloop; it++) {
		tdiffmtrx(arc, tprob, tdif1, tdif2);
		lkld1 = lkld2 = 0.0;
		for (k = 0; k < Numptrn; k++) {
			if ((j = dseqi[k]) >= 0) {
				opb = oprob[k];
				sumlk = sumd1 = sumd2 = 0.0;
				for (i = 0; i < TPMRADIX; i++) {
					prod = Freqtpm[i] * opb[i];
					sumlk += prod * tprob[i][j];
					sumd1 += prod * tdif1[i][j];
					sumd2 += prod * tdif2[i][j];
				}
				sumd1 /= sumlk;
				lkld1 += sumd1 * Weight[k];
				lkld2 += (sumd2 / sumlk - sumd1 * sumd1) * Weight[k];
			}
		}
		vari = 1.0 / fabs(lkld2);
		arcold = arc;
		arcdiff = - (lkld1 / lkld2);
		arc += arcdiff;
		if (arc > MIDARC && arcpre < 10.0) arc = MINARC;
		if (arc < MINARC) arc = MINARC;
		if (arc > MAXARC) arc = MAXARC;
		if (fabs(arcold - arc) < EPSILON) break;
	}
	op->kinp->length = arc;
	op->length = arc;
	op->lklhdl = vari;
	oprob = op->iprob;
	tprobmtrx(op->length, tprob);
	for (k = 0; k < Numptrn; k++) {
	opb = oprob[k];
		if ((j = dseqi[k]) >= 0) {
			for (i = 0; i < TPMRADIX; i++)
				opb[i] = tprob[i][j];
		} else {
			for (i = 0; i < TPMRADIX; i++)
				opb[i] = 1.0; /* !? */
		}
	}
}


void initpartlkl(Tree *tr)
{
	Node *cp, *rp;


	cp = rp = tr->rootp;
	do {
		cp = cp->isop->kinp;
		if (cp->isop == NULL) { /* external node */			
			cp = cp->kinp; /* not descen */
			partelkl(cp);
		} else { /* internal node */	
			if (!cp->descen) {
				prodpart(cp->kinp->isop);
				partilkl(cp);
			}
		}
	} while (cp != rp);
}


void mlikelihood(Tree *tr)
{
	Node *cp, *rp;
	int l, nconv, nconv2;
	double lendiff;


	nconv = nconv2 = 0;
	Numit = 0;
	Converg = FALSE;
	for (l = 0; l < MAXIT; l++) {
		Numit++;
		cp = rp = tr->rootp;
		do {
			cp = cp->isop->kinp;
			prodpart(cp->kinp->isop);
			if (cp->isop == NULL) { /* external node */	
				cp = cp->kinp; /* not descen */
				lendiff = cp->length;
				mlebranch(cp, l);
				lendiff = fabs(lendiff - cp->length);
				lendiff < EPSILON ? (nconv++)  : (nconv = 0);
				lendiff < 0.1     ? (nconv2++) : (nconv2 = 0);
			} else { /* internal node */
				if (cp->descen) {
					partilkl(cp);
				} else {
					lendiff = cp->length;
					mlibranch(cp, l);
					lendiff = fabs(lendiff - cp->length);
					lendiff < EPSILON ? (nconv++)  : (nconv = 0);
					lendiff < 0.1     ? (nconv2++) : (nconv2 = 0);
				}
			}
			if (nconv >= NUMBRNCH) goto convergence;
		} while (cp != rp);
	}
	if (nconv2 >= NUMBRNCH) Converg = 2;
	return;

convergence:
	Converg = TRUE;
	return;
}


/***************************** exported functions *****************************/


/* compute precise likelihood value */
void truelkl(Tree *tr)
{	
	Converg = TRUE;
	Numit = 0;
	
	initpartlkl(tr);
	mlikelihood(tr);
}


/* compute approximate likelihood value */
void aproxlkl(Tree *tr)
{
	int i, k;
	Node *cp, *rp;
	double sumlk, lklhd;
	dmatrix prob1, prob2, prob3;


	cp = rp = tr->rootp;
	do {
		cp = cp->isop->kinp;
		if (cp->isop == NULL) { /* external node */
			cp = cp->kinp; /* not descen */
			partelkl(cp);
		} else { /* internal node */
			if (!cp->descen) {
				prodpart(cp->kinp->isop);
				partilkl(cp);
			}
		}
	} while (cp != rp);
	lklhd = 0.0;
	 /* for binary tree only */
		prob1 = cp->iprob;
		prob2 = cp->isop->iprob;
		prob3 = cp->isop->isop->iprob;
		for (k = 0; k < Numptrn; k++) {
			sumlk = 0.0;
			for (i = 0; i < TPMRADIX; i++)
				sumlk += Freqtpm[i] * prob1[k][i] * prob2[k][i] * prob3[k][i];
			lklhd += log(sumlk) * Weight[k];
		}
	
	tr->lklhd = lklhd;
	return;
}


/******************************************************************************/
/* mltree.c  computation of quartet maximum likelihood                        */
/******************************************************************************/


/***************************** internal functions *****************************/


void luequation(dmatrix amat, dvector yvec, int size)
{
    double eps = 1.0e-20; /* ! */
	int i, j, k, l, maxi, idx;
	double sum, tmp, maxb, aw;
	dvector wk;
	ivector index;


	wk = new_dvector(size);
	index = new_ivector(size);
	aw = 1.0;
	for (i = 0; i < size; i++) {
		maxb = 0.0;
		for (j = 0; j < size; j++) {
			if (fabs(amat[i][j]) > maxb)
				maxb = fabs(amat[i][j]);
		}
		if (maxb == 0.0) {
			fprintf(stderr, "\nERROR: Singular matrix [luequation]\n");
			exit(1);
		}
		wk[i] = 1.0 / maxb;
	}
	for (j = 0; j < size; j++) {
		for (i = 0; i < j; i++) {
			sum = amat[i][j];
			for (k = 0; k < i; k++)
				sum -= amat[i][k] * amat[k][j];
			amat[i][j] = sum;
		}
		maxb = 0.0;
		for (i = j; i < size; i++) {
			sum = amat[i][j];
			for (k = 0; k < j; k++)
				sum -= amat[i][k] * amat[k][j];
			amat[i][j] = sum;
			tmp = wk[i] * fabs(sum);
			if (tmp >= maxb) {
				maxb = tmp;
				maxi = i;
			}
		}
		if (j != maxi) {
			for (k = 0; k < size; k++) {
				tmp = amat[maxi][k];
				amat[maxi][k] = amat[j][k];
				amat[j][k] = tmp;
			}
			aw = -aw;
			wk[maxi] = wk[j];
		}
		index[j] = maxi;
		if (amat[j][j] == 0.0)
			amat[j][j] = eps;
		if (j != size - 1) {
			tmp = 1.0 / amat[j][j];
			for (i = j + 1; i < size; i++)
				amat[i][j] *= tmp;
		}
	}
	l = -1;
	for (i = 0; i < size; i++) {
		idx = index[i];
		sum = yvec[idx];
		yvec[idx] = yvec[i];
		if (l != -1) {
			for (j = l; j < i; j++)
				sum -= amat[i][j] * yvec[j];
		} else if (sum != 0.0)
			l = i;
		yvec[i] = sum;
	}
	for (i = size - 1; i >= 0; i--) {
		sum = yvec[i];
		for (j = i + 1; j < size; j++)
			sum -= amat[i][j] * yvec[j];
		yvec[i] = sum / amat[i][i];
	}
	free_ivector(index);
	free_dvector(wk);
}


/* least square estimation for branch lengths
   used for approximate likelihood and as starting point 
   in the calculation of the exact value of the likelihood */

void lslength(Tree *tr, dvector distanvec, int numspc)
{
	int i, i1, j, j1, j2, k, numibrnch, numbrnch, numpair;
	double sum, leng, alllen, rss;
	ivector pths;
	dmatrix atmt, atamt;
	Node **ebp, **ibp;


	numibrnch = NUMIBRNCH;
	numbrnch = numspc + numibrnch;
	numpair = (numspc * (numspc - 1)) / 2;
	atmt = new_dmatrix(numbrnch, numpair);
	atamt = new_dmatrix(numbrnch, numbrnch);
	ebp = tr->ebrnchp;
	ibp = tr->ibrnchp;
	for (i = 0; i < numspc; i++) {
		for (j1 = 1, j = 0; j1 < numspc; j1++) {
			if (j1 == i) {
				for (j2 = 0; j2 < j1; j2++, j++) {
					atmt[i][j] = 1.0;
				}
			} else {
				for (j2 = 0; j2 < j1; j2++, j++) {
					if (j2 == i)
						atmt[i][j] = 1.0;
					else
						atmt[i][j] = 0.0;
				}
			}
		}
	}
	for (i1 = 0, i = numspc; i1 < numibrnch; i1++, i++) {
		pths = ibp[i1]->paths;
		for (j1 = 1, j = 0; j1 < numspc; j1++) {
			for (j2 = 0; j2 < j1; j2++, j++) {
				if (pths[j1] != pths[j2])
					atmt[i][j] = 1.0;
				else
					atmt[i][j] = 0.0;
			}
		}
	}
	for (i = 0; i < numbrnch; i++) {
		for (j = 0; j <= i; j++) {
			for (k = 0, sum = 0.0; k < numpair; k++)
				sum += atmt[i][k] * atmt[j][k];
			atamt[i][j] = sum;
			atamt[j][i] = sum;
		}
	}
	for (i = 0; i < numbrnch; i++) {
		for (k = 0, sum = 0.0; k < numpair; k++)
			sum += atmt[i][k] * distanvec[k];
		Brnlength[i] = sum;
	}
	luequation(atamt, Brnlength, numbrnch);
	for (i = 0, rss = 0.0; i < numpair; i++) {
		sum = distanvec[i];
		for (j = 0; j < numbrnch; j++) {
			if (atmt[j][i] == 1.0 && Brnlength[j] > 0.0)
				sum -= Brnlength[j];
		}
		rss += sum * sum;
	}
	alllen = 0.0;
	for (i = 0; i < numspc; i++) {
		leng = Brnlength[i];
		alllen += leng;
		if (leng < MINARC) leng = MINARC;
		if (leng > MAXARC) leng = MAXARC;
		ebp[i]->length = leng;
		ebp[i]->kinp->length = leng;
		Brnlength[i] = leng;
	}
	for (i = 0, j = numspc; i < numibrnch; i++, j++) {
		leng = Brnlength[j];
		alllen += leng;
		if (leng < MINARC) leng = MINARC;
		if (leng > MAXARC) leng = MAXARC;
		ibp[i]->length = leng;
		ibp[i]->kinp->length = leng;
		Brnlength[j] = leng;
	}
	free_dmatrix(atmt);
	free_dmatrix(atamt);
}


/* make (a,b)-(c,d) quartet

	a ---+     +--- c
	     +-----+
	b ---+     +--- d

	species numbers range from 0 to Maxspc - 1  */

void make_quartet(int a, int b, int c, int d)
{
	/* place sequences */
	Ctree->ebrnchp[0]->eprob = Seqconint[a];
	Ctree->ebrnchp[1]->eprob = Seqconint[b];
	Ctree->ebrnchp[2]->eprob = Seqconint[c];
	Ctree->ebrnchp[3]->eprob = Seqconint[d];
	
	/* make distance vector */
	Distanvec[0] = Distanmat[b][a];
	Distanvec[1] = Distanmat[c][a];
	Distanvec[2] = Distanmat[c][b];
	Distanvec[3] = Distanmat[d][a];
	Distanvec[4] = Distanmat[d][b];
	Distanvec[5] = Distanmat[d][c];
}


/* initialise quartet */
Tree *new_quartet(int numptrn, imatrix seqconint)
{
	int n, i;
	Tree *tr;
	Node *dp, *up;

	
	/* reserve memory for tree */
	tr = (Tree *) malloc(sizeof(Tree));
	if (tr == NULL) maerror("new_quartet [tr]");
	tr->ebrnchp = (Node **) malloc((unsigned) 4 * sizeof(Node *));
	if (tr->ebrnchp == NULL) maerror("new_quartet [ebrnchp]");
	tr->ibrnchp = (Node **) malloc((unsigned) sizeof(Node *));
	if (tr->ibrnchp == NULL) maerror(" new_quartet [ibrnchp]");

	/* reserve memory for nodes */
	for (n = 0; n < 4; n++) {
		dp = (Node *) malloc(sizeof(Node));
		if (dp == NULL) maerror("new_quartet [dp, external branch]");
		up = (Node *) malloc(sizeof(Node));
		if (up == NULL) maerror("new_quartet [up, external branch]");
		dp->isop = NULL;
		dp->kinp = up;
		up->kinp = dp;
		dp->descen = TRUE;
		up->descen = FALSE;
		dp->number = n;
		up->number = n;
		dp->length = 0.0;
		up->length = 0.0;
		dp->lklhdl = 0.0;
		up->lklhdl = 0.0;
		dp->paths = new_ivector(4);
		up->paths = dp->paths;
		for (i = 0; i < 4; i++) dp->paths[i] = 0;
		dp->paths[n] = 1;
		dp->eprob = seqconint[n]; /* make quartet (0,1)-(2,3) as default */
		up->eprob = NULL;
		dp->iprob = NULL;
		up->iprob = new_dmatrix(numptrn, TPMRADIX);
		tr->ebrnchp[n] = dp;
	}

	/* reserve memory for internal branch */	
	dp = (Node *) malloc(sizeof(Node));
	if (dp == NULL) maerror("new_quartet [dp, internal branch]");	
	up = (Node *) malloc(sizeof(Node));
	if (up == NULL) maerror("new_quartet [up, internal branch]");	
	dp->isop = tr->ebrnchp[3]->kinp; /* connect internal branch */
	up->isop = tr->ebrnchp[0]->kinp;
	dp->kinp = up;
	up->kinp = dp;
	dp->descen = TRUE;
	up->descen = FALSE;
	dp->number = 0;
	up->number = 0;
	dp->length = 0.0;
	up->length = 0.0;
	dp->lklhdl = 0.0;
	up->lklhdl = 0.0;
	dp->paths = new_ivector(4);
	up->paths = dp->paths;
	up->paths[0] = 0;
	up->paths[1] = 0;
	up->paths[2] = 1;
	up->paths[3] = 1;
	dp->eprob = NULL;
	up->eprob = NULL;
	dp->iprob = new_dmatrix(numptrn, TPMRADIX);
	up->iprob = new_dmatrix(numptrn, TPMRADIX);
	tr->ibrnchp[0] = dp;
	
	/* place root */
	tr->rootp = up;

	/* connect external branches */ 
	tr->ebrnchp[0]->kinp->isop = tr->ebrnchp[1]->kinp;
	tr->ebrnchp[1]->kinp->isop = tr->rootp;
	tr->ebrnchp[3]->kinp->isop = tr->ebrnchp[2]->kinp;
	tr->ebrnchp[2]->kinp->isop = tr->rootp->kinp;
	
	/*
	 * reserve memory for lengths of the five branches
	 * of a quartet and for the six possible distances
	 * (needed for LS estimation of branch lengths)
	 */
	Brnlength = new_dvector(5); 
	Distanvec = new_dvector(6);

	return tr;
}


/* free quartet memory */
void free_quartet(Tree *tr)
{	
	int n;
	Node *dp, *up;


	for (n = 0; n < NUMSPC; n++) {
		dp = tr->ebrnchp[n];
		up = dp->kinp;
		free_ivector(dp->paths);
		free_dmatrix(up->iprob);
		free(dp);
		free(up);
	}
	free(tr->ebrnchp);
	for (n = 0; n < NUMIBRNCH; n++) {
		dp = tr->ibrnchp[n];
		up = dp->kinp;
		free_dmatrix(dp->iprob);
		free_dmatrix(up->iprob);
		free_ivector(dp->paths);
		free(dp);
		free(up);
	}
	free(tr->ibrnchp);
	free(tr);
	free_dvector(Brnlength); /* branch lengths of quartet (for LS estimation) */
	free_dvector(Distanvec); /* distances in quartet (for LS estimation) */
}


void openfiles()
{	
	char inputline[64];
	
	
	/* open sequencefile  */
    if ((seqfp = fopen(INFILE, "r")) == NULL) {
   		fprintf(stderr,
   			"\n\nERROR: I can't open the sequence input file: %s\n", INFILE);
		fprintf(stderr, "Please enter another name for this file: ");
		gets(inputline);
		if ((seqfp = fopen(inputline, "r")) == NULL) {
			fprintf(stderr,
				"\nERROR: I can't open this file either!\n");
			exit(1);
		}
		fprintf(stderr, "\n");
	}	
}


void closefiles()
{
	fclose(seqfp);
}


/***************************** exported functions *****************************/


/* compute the likelihood for (a,b)-(c,d) quartet */
double quartet_lklhd(int a, int b, int c, int d)
{
	/* make (a,b)-(c,d) quartet */
	make_quartet(a,b,c,d);
	
	/* least square estimate for branch length */	
	lslength(Ctree, Distanvec, NUMSPC);

	/* compute likelihood */
	truelkl(Ctree);

	return Ctree->lklhd;
}


/* compute the approximate likelihood for (a,b)-(c,d) quartet */
double quartet_alklhd(int a, int b, int c, int d)
{
	/* make (a,b)-(c,d) quartet */
	make_quartet(a,b,c,d);
	
	/* least square estimate for branch length */	
	lslength(Ctree, Distanvec, NUMSPC);

	/* compute likelihood */
	aproxlkl(Ctree);

	return Ctree->lklhd;
}


/* preparation for quartet ML analysis */
void qmlstart()
{	
	/* open files */
	openfiles();
	
	/* look at first line of sequence input file */
	getsizesites(seqfp);
	
	/* get the sequences */
	getdataset(seqfp);
	
	/* process sequence information */
	evaluateseqs();
	
	/* compute transition probability matrix */	
	tranprobmat();

	/* compute distance matrix */
	computedistan();

	/* initialise memory for quartet (need to be called only once) */
	Ctree = new_quartet(Numptrn, Seqconint);
}


/* cleanup after quartet ML analysis */
void qmlfinish()
{
	closefiles();
	free_quartet(Ctree);
	free_imatrix(Seqconint); /* from seqproc */
	free_ivector(Weight);
	free_dmatrix(Distanmat); /* from distan */
	free_cmatrix(Identif); /* from getseq */
	free_cmatrix(Seqchar);
}


/******************************************************************************/
/* end of qml.c                                                               */
/******************************************************************************/
